#include <map>
#include <set>
#include <cstdio>
#include <vector>
#include <algorithm>
using namespace std;

const int MAXN = 1 << 16;

struct node {
	node *c[2];
	node *p;
	int val;
	int num;
	
	node () {
		c[0] = c[1] = p = NULL;
		num = 0;
	}
	
	void upd() {
		num = 1;
		if (c[0]) num += c[0]->num;
		if (c[1]) num += c[1]->num;
	}
	
	void sc(node *t, int d) {
		c[d] = t;
		if(t) t->p = this;
	}
	
    void rot() {
        int t = p->c[1] == this;
        node *y = p;
        if(y->p) y->p->sc(this, y->p->c[1] == y);
        p = y->p;
        y->sc(c[t ^ 1], t);
        sc(y, t ^ 1);
        c[t ^ 1]->upd();
        upd();
    }
    
	node* splay() {
		while (p != NULL) {
			if (p->p == NULL) {
				rot();
			} else {
				if((p->c[0] == this) ^ (p->p->c[0] == p)) {
					rot();
					rot();
				} else {
					p->rot();
					rot();
				}
			}
		}
        
        return this;
	}
};

struct tree {
    node *root;
    int l, r;
    
    tree () {
        root = NULL;
    }
};

int n, q;
int a[MAXN];
int b[MAXN];
tree it[MAXN * 2];
int m;

map<int, set<int> > mp;

void read() {
    scanf("%d", &n);
    for (int i = 0; i < n; i++) {
        scanf("%d", &a[i]);
    }
    
    scanf("%d%d", &q, &m);
}

void init(int p, int l, int r) {
    it[p].l = l;
    it[p].r = r;
    
    vector<int> w;
    
    for (int i = l; i <= r && i < n; i++) {
        w.push_back(b[i]);
    }
    sort(w.rbegin(), w.rend());
    for (int i = 0; i < (int)w.size(); i++) {
        node *q = new node;
        q->val = w[i];
        q->c[0] = NULL;
        q->sc(it[p].root, 1);
        q->num = 1;
        
        if (it[p].root != NULL) {
            q->num += it[p].root->num;
        }
        
        it[p].root = q;
    }
    
    if (l != r) {
        int mid = (l + r) / 2;
        init(p * 2, l, mid);
        init(p * 2 + 1, mid + 1, r);
    }
}

int get(node *&r, int val) {
    while (1) {
        if (r->val < val) {
            if (r->c[1] == NULL) break;
            r = r->c[1];
        } else {
            if (r->c[0] == NULL) break;
            r = r->c[0];
        }
    }
    r->splay();
    int ans = 0;
    
    if (r->val < val) {
        ans = 1;
    }
    
    if (r->c[0]) {
        ans += r->c[0]->num;
    }
    return ans;
}

void remnode(node *&p, int x) {
    get(p, x);
    if (p->val != x) {
        int d;
        if (p->val > x) {
            d = 0;
        } else {
            d = 1;
        }
        p = p->c[d];
        while (p->c[d^1]) p = p->c[d^1];
        p->splay();
    }
    node *l = p->c[0];
    node *r = p->c[1];
    
    if (l) l->p = NULL;
    if (r) r->p = NULL;
    delete p;
    
    if(l) {
        while (l->c[1]) l = l->c[1];
        l = l->splay();
        l->sc(r, 1);
        if(r) l->num += r->num;
        
        p = l;
    } else {
        if (r) {
            p = r;
        } else {
            p = NULL;
        }
    }
}

void addnode(node *&p, int x) {
    node *t = new node;
    t->val = x;
    t->num = 1;
    if (p == NULL) {
        p = t;
        return ;
    }
    
    get(p, x);
    int d;
    if (p->val <= x) {
        d = 1;
    } else {
        d = 0;
    }
    
    t->sc(p->c[d], d);
    p->sc(t, d);
    if (t->c[d]) {
        t->num += t->c[d]->num;
    }
    p->num ++;
}

int ask(int p, int l, int r) {
    if (it[p].r < l || it[p].l > r) return 0;
    if (l <= it[p].l && it[p].r <= r) {
        return get(it[p].root, l);
    }
    
    return ask(p * 2, l , r) + ask(p * 2 + 1, l, r);
}

void rem(int p, int x, int y) {
    if (it[p].r < x || x < it[p].l) return;
    remnode(it[p].root, y);
    if (it[p].l == it[p].r) return;
    
    rem(p * 2, x, y);
    rem(p * 2 + 1, x, y);
}

void add(int p, int x, int y) {
    if (it[p].r < x || x < it[p].l) return;
    addnode(it[p].root, y);
    
    if (it[p].l == it[p].r) return;
    
    add(p * 2, x, y);
    add(p * 2 + 1, x, y);
}

void upd(int x, int y) {
    rem(1, x, b[x]);
    b[x] = y;
    add(1, x, y);
}

void solve() {
    for (int i = 0; i < n; i++) {
        if (mp[ a[i] ].size() == 0) {
            b[i] = -1;
        } else {
            multiset<int>::reverse_iterator it = mp[ a[i] ].rbegin();
            b[i] = (*it);
        }
        mp[ a[i] ].insert(i);
    }
    
    int p = 1;
	int R = 0;
    while (p < n) p *= 2;
    init(1, 0, p - 1);
    
    while (q --) {
        int x, y, z;
        scanf ("%d%d%d", &x, &y, &z);
		if (m) {
			y ^= R;
			z ^= R;
		}
        
        if (x == 0) {
            printf ("%d\n", R = ask(1, y, z));
        } else {
            multiset<int>::iterator q, w;
            int left, righ;
            q = w = mp[ a[y] ].lower_bound(y);
            
            if (q == mp[ a[y] ].begin()) {
                left = -1;
            } else {
                -- q;
                left = *q;
            }
            
            ++ w;
            if (w == mp[ a[y] ].end()) {
                righ = -1;
            } else {
                righ = *w;
            }
            
            if (righ != -1) {
                upd(righ, left);
            }
            mp[ a[y] ].erase(y);
            //erased
            
            a[y] = z;
            mp[z].insert(y);
            q = w = mp[ a[y] ].lower_bound(y);
            
            if (q == mp[ a[y] ].begin()) {
                left = -1;
            } else {
                -- q;
                left = *q;
            }
            
            ++ w;
            if (w == mp[ a[y] ].end()) {
                righ = -1;
            } else {
                righ = *w;
            }
            
            if (righ != -1) {
                upd(righ, y);
            }
            upd(y, left);
        }
    }
}

int main() {
    read();
    solve();
    
    return 0;
}